﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.ObjectModel;
using System.ComponentModel;
using BizValidationSample.Entities;
using System.Windows.Input;
using System.Diagnostics;

namespace BizValidationSample
{
    public class ActivityEditorViewModel : INotifyPropertyChanged
    {
        public ActivityEditorViewModel()
        {
            IActivitiesRepository repository = new ActivitiesRepository();
            Objectives = new ObservableCollection<Objective>(repository.Objectives());
            Customers = new ObservableCollection<Customer>(repository.Customers());
            ActivityTypes = new ObservableCollection<ActivityType>(repository.ActivityTypes());
            AddCommand = new DelegateCommand<object>(OnAdd, OnCanAdd);
            CancelCommand = new DelegateCommand<object>(OnCancel);
            Activity = new Activity();
            Activity.ActivityType = ActivityTypes[0];
            Activity.Objective = Objectives.FirstOrDefault(o=>o.Name=="N/A");
            Activity.Customer = Customers.FirstOrDefault(c=>c.Name=="N/A");
        }
        public ObservableCollection<Objective> Objectives { get; private set; }
        public ObservableCollection<Customer> Customers { get; private set; }
        public ObservableCollection<ActivityType> ActivityTypes { get; private set; }
        public DelegateCommand<object> AddCommand { get; set; }
        public ICommand CancelCommand { get; set; }

        Activity _Activity;
        public Activity Activity
        {
            get
            {
                return _Activity;
            }
            set
            {
                if (value != _Activity)
                {
                    if (_Activity != null) _Activity.PropertyChanged -= OnActivityChanged;
                    _Activity = value;
                    if (_Activity != null) _Activity.PropertyChanged += OnActivityChanged;
                    PropertyChanged(this, new PropertyChangedEventArgs("Activity"));
                }
            }
        }

        private void OnActivityChanged(object sender, PropertyChangedEventArgs e)
        {
            AddCommand.RaiseCanExecuteChanged();
        }
        private void OnAdd(object obj)
        {
            
        }

        private bool OnCanAdd(object arg)
        {
            if (Activity != null) return Activity.IsValid();
            return false;
        }

        private void OnCancel(object obj)
        {
            Debug.WriteLine("Adding cancelled");
            Activity = new Activity();
        }
        public event PropertyChangedEventHandler PropertyChanged = delegate { };

    }
}
