﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;

namespace BizValidationSample.Entities
{
    public class Activity : INotifyPropertyChanged, IDataErrorInfo
    {
        public Activity()
        {
            ActivityDate = DateTime.Now;
        }
        int _ActivityId;
        public int ActivityId
        {
            get
            {
                return _ActivityId;
            }
            set
            {
                if (value != _ActivityId)
                {
                    _ActivityId = value;
                    PropertyChanged(this, new PropertyChangedEventArgs("ActivityId"));
                }
            }
        }

        string _Title;
        public string Title
        {
            get
            {
                return _Title;
            }
            set
            {
                if (value != _Title)
                {
                    _Title = value;
                    PropertyChanged(this, new PropertyChangedEventArgs("Title"));
                }
            }
        }

        string _Description;
        public string Description
        {
            get
            {
                return _Description;
            }
            set
            {
                if (value != _Description)
                {
                    if (string.IsNullOrWhiteSpace(value))
                        throw new ArgumentException("Description must not be empty");
                    _Description = value;
                    PropertyChanged(this, new PropertyChangedEventArgs("Description"));
                }
            }
        }

        ActivityType _ActivityType;
        public ActivityType ActivityType
        {
            get
            {
                return _ActivityType;
            }
            set
            {
                if (value != _ActivityType)
                {
                    _ActivityType = value;
                    PropertyChanged(this, new PropertyChangedEventArgs("ActivityType"));
                    PropertyChanged(this, new PropertyChangedEventArgs("Inventory"));
                }
            }
        }

        DateTime _ActivityDate;
        public DateTime ActivityDate
        {
            get
            {
                return _ActivityDate;
            }
            set
            {
                if (value != _ActivityDate)
                {
                    _ActivityDate = value;
                    PropertyChanged(this, new PropertyChangedEventArgs("ActivityDate"));
                }
            }
        }


        string _Inventory;
        public string Inventory
        {
            get
            {
                return _Inventory;
            }
            set
            {
                if (value != _Inventory)
                {
                    _Inventory = value;
                    PropertyChanged(this, new PropertyChangedEventArgs("Inventory"));
                }
            }
        }

        Customer _Customer;
        public Customer Customer
        {
            get
            {
                return _Customer;
            }
            set
            {
                if (value != _Customer)
                {
                    _Customer = value;
                    PropertyChanged(this, new PropertyChangedEventArgs("Reason"));
                    PropertyChanged(this, new PropertyChangedEventArgs("Objective"));
                    PropertyChanged(this, new PropertyChangedEventArgs("Customer"));
                }
            }
        }

        Objective _Objective;
        public Objective Objective
        {
            get
            {
                return _Objective;
            }
            set
            {
                if (value != _Objective)
                {
                    _Objective = value;
                    PropertyChanged(this, new PropertyChangedEventArgs("Reason"));
                    PropertyChanged(this, new PropertyChangedEventArgs("Objective"));
                    PropertyChanged(this, new PropertyChangedEventArgs("Customer"));
                }
            }
        }

        string _Reason;
        public string Reason
        {
            get
            {
                return _Reason;
            }
            set
            {
                if (value != _Reason)
                {
                    _Reason = value;
                    PropertyChanged(this, new PropertyChangedEventArgs("Reason"));
                    PropertyChanged(this, new PropertyChangedEventArgs("Objective"));
                    PropertyChanged(this, new PropertyChangedEventArgs("Customer"));
                }
            }
        }

        public event PropertyChangedEventHandler PropertyChanged = delegate { };

        public string Error
        {
            get { return null; }
        }

        public string this[string propertyName]
        {
            get { return IsValid(propertyName); }
        }

        public bool IsValid()
        {
            return string.IsNullOrEmpty(IsValid("Title")) && 
                string.IsNullOrEmpty(IsValid("Description")) &&
                string.IsNullOrEmpty(IsValid("Inventory")) &&
                string.IsNullOrEmpty(IsValid("Customer")) &&
                string.IsNullOrEmpty(IsValid("Objective")) &&
                string.IsNullOrEmpty(IsValid("Reason")) &&
                string.IsNullOrEmpty(IsValid("ActivityDate"));
        }

        private string IsValid(string propertyName)
        {
            switch (propertyName)
            {
                case "Title":
                    if (string.IsNullOrWhiteSpace(Title))
                        return "Title is required";
                    break;
                case "Description":
                    if (string.IsNullOrWhiteSpace(Description))
                        return "Description is required";
                    break;
                case "Inventory":
                    if (ActivityType != null && ActivityType.Name == "Install" && string.IsNullOrWhiteSpace(Inventory))
                        return "Inventory expended must be entered for installs";
                    break;
                case "ActivityDate":
                    if (!IsValidDate(ActivityDate))
                        return "Activity date must be for the current week";
                    break;
                case "Reason":
                case "Objective":
                case "Customer":
                    if (!CustomerObjectiveOrReasonEntered())
                        return "At least one of Customer, Objective, or Reason must be provided";
                    break;
                default:
                    break;
            }
            return null;
        }
        // No more than three days back or forward
        private bool IsValidDate(DateTime activityDate)
        {
            DateTime today = DateTime.Today;
            if ((activityDate.Date - today).Days > 3)
                return false;
            if ((today - activityDate.Date).Days > 3)
                return false;

            return true;
        }
        public bool CustomerObjectiveOrReasonEntered()
        {
            if ((Customer == null ||  Customer.Name=="N/A") && (Objective == null || Objective.Name=="N/A") && string.IsNullOrWhiteSpace(Reason))
                return false;
            return true;
        }

    }
}
